""" Define and retrieve all command line options.
"""

import argparse


def get_command_line_arguments(version: str):
    """build, parse and return command line arguments

    Args:
        version (str): the current program version

    Returns:
        the parsed command line arguments
    """

    program_name = "mrcsmooth"
    description = ("Uses a filter on a three dimensional density data "
                   "to reduce the number of voxels by 2 for each pass, "
                   "effectively smoothening the data while reducing "
                   "the data size.")

    parser = argparse.ArgumentParser(
        description=description,
        prog=program_name,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument("--density",
                        required=True,
                        help="input mrc density filename",
                        metavar="input.mrc")

    parser.add_argument("--output-density",
                        nargs='?',
                        default='smooth.mrc',
                        help="file to write the fitted structure to.",
                        metavar="smooth.mrc")

    parser.add_argument("--passes",
                        type=int,
                        default=1,
                        help="how often to apply the filter, "
                             "however have at least 4 voxels "
                             "per dimension in the output data.")

    parser.add_argument("--version",
                        action="version",
                        version=(f"{program_name} {version}"))

    return parser.parse_args()



""" Run the command line interface """
import logging
import shutil

import importlib.metadata

import mrcfile

import scipy.signal

import numpy as np

import rich.console
import rich.logging

import mrcsmooth.arguments


def create_rich_logger():
    FORMAT = "%(message)s"
    logging.basicConfig(level="INFO",
                        format=FORMAT,
                        datefmt="[%X]",
                        handlers=[rich.logging.RichHandler()])
    return logging.getLogger("rich")


def run():
    """ run the command line interface """

    # set up the console for printing and logging
    console = rich.console.Console()
    log = create_rich_logger()

    # derive the program version via git
    try:
        version = importlib.metadata.version("mrcsmooth")
    except importlib.metadata.PackageNotFoundError:
        version = "Unknown"

    command_line_arguments = (
        mrcsmooth.arguments.get_command_line_arguments(version))

    log.info("Reading density ...")

    # because copying mrcfile objects is hard, start out by copying the file
    shutil.copy2(command_line_arguments.density,
                 command_line_arguments.output_density)

    output_density = mrcfile.open(command_line_arguments.output_density,
                                  'r+',
                                  permissive=True)

    output_density.header.ispg = mrcfile.constants.VOLUME_SPACEGROUP

    resampled_data = output_density.data
    for resampling_step in range(command_line_arguments.passes):
        if min(resampled_data.shape) < 8:
            log.info(f"Stopping resampling at pass {resampling_step} despite "
                     f"the requested {command_line_arguments.passes} to "
                     f"have a meaningful density left.")
            break
        for axis in range(3):
            resampled_data = scipy.signal.resample(
                resampled_data, resampled_data.shape[axis] // 2, axis=axis)

    output_density.set_data(resampled_data)

    output_density.close()

    log.info("done")
